@@ -1,4 +1,5 @@
-from typing import Dict, List, Tuple, Union
+from itertools import product
+from typing import Dict, List, Union
 
 import numexpr as ne
 import numpy as np
@@ -32,21 +33,51 @@ class CarModel:
     def __init__(
         self,
         array: xr.DataArray,
+        country="RER",
         cycle: Union[None, str, np.ndarray] = None,
         gradient: Union[None, np.ndarray] = None,
         energy_storage: Union[None, Dict] = None,
+        electric_utility_factor: float = None,
+        drop_hybrids: bool = True,
+        energy_consumption: dict = None,
+        target_range: dict = None,
+        target_mass: dict = None,
+        power: dict = None,
     ) -> None:
 
         self.array = array
+        self.country = country
 
         if cycle is None:
             self.ecm = EnergyConsumptionModel("WLTC")
         else:
             self.ecm = EnergyConsumptionModel(cycle=cycle, gradient=gradient)
 
-        self.energy_storage = None
-        self.set_battery_preferences(energy_storage)
+        # override default values for batteries
+        # if provided by the user
+        self.energy_storage = {
+            "electric": {
+                x: "NMC-622"
+                for x in product(
+                    ["BEV", "PHEV-e", "HEV-d", "HEV-p"],
+                    self.array.coords["size"].values,
+                    self.array.year.values,
+                )
+            }
+        }
+        self.energy_storage["origin"] = "CN"
+        self.energy_storage.update(energy_storage or {})
+        self.set_battery_preferences()
         self.energy = None
+        self.electric_utility_factor = electric_utility_factor
+        self.drop_hybrids = drop_hybrids
+        self.energy_consumption = energy_consumption or None
+        # a range to reach cna be defined by the
+        self.target_range = target_range
+        # a curb mass to reach can be defined by the user
+        self.target_mass = target_mass
+        # overrides the engine/motor power
+        self.power = power
 
     def __call__(self, key: Union[str, List]):
 
@@ -96,7 +127,7 @@ class CarModel:
     def __setitem__(self, key, value):
         self.array.loc[{"parameter": key}] = value
 
-    def set_all(self, drop_hybrids: bool = True, electric_utility_factor: float = None):
+    def set_all(self):
         """
         This method runs a series of other methods to obtain the tank-to-wheel energy requirement, efficiency
         of the car, costs, etc.
@@ -123,7 +154,6 @@ class CarModel:
             old_driving_mass = self["driving mass"].sum().values
 
             self.set_car_masses()
-
             self.set_power_parameters()
             self.set_component_masses()
             self.set_battery_properties()
@@ -141,15 +171,22 @@ class CarModel:
         self.calculate_ttw_energy()
         self.set_share_recuperated_energy()
         self.adjust_cost()
+
+        # if user-provided values are passed,
+        # they override the default values
+        if self.target_range or self.energy_storage:
+            self.set_storage_size_or_range()
+            self.set_energy_stored_properties()
+
         self.set_range()
-        self.set_electric_utility_factor(electric_utility_factor)
+        self.set_electric_utility_factor()
         self.set_electricity_consumption()
         self.set_costs()
         self.set_hot_emissions()
         self.set_particulates_emission()
         self.set_noise_emissions()
         self.create_PHEV()
-        if drop_hybrids:
+        if self.drop_hybrids:
             self.drop_hybrid()
 
         # we flag cars that have a range inferior to 100 km
@@ -230,20 +267,36 @@ class CarModel:
                     )
                 ]
 
-    def set_battery_preferences(self, energy_storage):
+    def set_battery_preferences(self):
 
-        self.energy_storage = energy_storage or {
-            "electric": {
-                "type": "NMC-622",
-            }
-        }
+        for key, val in self.energy_storage["electric"].items():
+            pwt, size, year = key
 
-        self["battery cell energy density"] = self[
-            f"battery cell energy density, {self.energy_storage['electric']['type'].split('-')[0].strip()}"
-        ]
-        self["battery cell mass share"] = self[
-            f"battery cell mass share, {self.energy_storage['electric']['type'].split('-')[0].strip()}"
-        ]
+            if val is not None and pwt in self.array.powertrain.values:
+
+                cell_params = self.array.loc[
+                    dict(
+                        powertrain=pwt,
+                        size=size,
+                        year=year,
+                        parameter=[
+                            f"battery cell energy density, {val.split('-')[0].strip()}",
+                            f"battery cell mass share, {val.split('-')[0].strip()}",
+                        ],
+                    )
+                ]
+
+                self.array.loc[
+                    dict(
+                        powertrain=pwt,
+                        size=size,
+                        year=year,
+                        parameter=[
+                            "battery cell energy density",
+                            "battery cell mass share",
+                        ],
+                    )
+                ] = cell_params.values
 
     def adjust_cost(self) -> None:
         """
@@ -529,6 +582,37 @@ class CarModel:
             / distance
         ).T
 
+        # override of TtW energy, provided by the user
+        if self.energy_consumption:
+            for key, val in self.energy_consumption.items():
+                pwt, size, year = key
+                if val is not None:
+
+                    print(
+                        f"Overriding TtW energy for {pwt} {size} {year} with {val} kj/km"
+                    )
+
+                    self.array.loc[
+                        dict(
+                            powertrain=pwt, size=size, year=year, parameter="TtW energy"
+                        )
+                    ] = val
+
+                    self.energy.loc[
+                        dict(
+                            powertrain=pwt,
+                            size=size,
+                            year=year,
+                            parameter="motive energy",
+                        )
+                    ] = (
+                        val  # kj/km
+                        * distance.mean()  # km
+                        / self.energy.shape[-1]  # seconds
+                    ).reshape(
+                        1, -1
+                    )
+
         self["TtW energy, combustion mode"] = self["TtW energy"] * (
             self["combustion power share"] > 0
         )
@@ -759,8 +843,42 @@ class CarModel:
             "battery BoP mass",
             "fuel tank mass",
         ]
+
         self["curb mass"] += self[curb_mass_includes].sum(axis=2)
 
+        if self.target_mass:
+            for key, target_mass in self.target_mass.items():
+                pwt, size, year = key
+
+                if target_mass:
+                    current_curb_mass = self.array.loc[
+                        dict(
+                            powertrain=pwt, size=size, year=year, parameter="curb mass"
+                        )
+                    ]
+                    mass_difference = target_mass - current_curb_mass
+
+                    lightweighting = self.array.loc[
+                        dict(
+                            powertrain=pwt,
+                            size=size,
+                            year=year,
+                            parameter="lightweighting",
+                        )
+                    ]
+
+                    self.array.loc[
+                        dict(
+                            powertrain=pwt,
+                            size=size,
+                            year=year,
+                            parameter="glider base mass",
+                        )
+                    ] += mass_difference / (1 - lightweighting)
+
+            self["curb mass"] = self["glider base mass"] * (1 - self["lightweighting"])
+            self["curb mass"] += self[curb_mass_includes].sum(axis=2)
+
         self["total cargo mass"] = (
             self["average passengers"] * self["average passenger mass"]
             + self["cargo mass"]
@@ -771,6 +889,15 @@ class CarModel:
         """Set electric and combustion motor powers based on input parameter ``power to mass ratio``."""
         # Convert from W/kg to kW
         self["power"] = self["power to mass ratio"] * self["curb mass"] / 1000
+
+        if self.power:
+            for key, power in self.power.items():
+                pwt, size, year = key
+                if power:
+                    self.array.loc[
+                        dict(powertrain=pwt, size=size, year=year, parameter="power")
+                    ] = power
+
         self["combustion power share"] = self["combustion power share"].clip(
             min=0, max=1
         )
@@ -812,37 +939,47 @@ class CarModel:
                 dict(powertrain="PHEV-e", parameter="share recuperated energy")
             ]
 
-    def set_electric_utility_factor(self, uf: float = None) -> None:
-        """Set the electric utility factor according to a sampled values in Germany (ICTT 2020)
-        https://theicct.org/sites/default/files/publications/PHEV-white%20paper-sept2020-0.pdf
+    def set_electric_utility_factor(self) -> None:
+        """Set the electric utility factor according to a sampled values in Germany (ICTT 2022)
+        https://theicct.org/wp-content/uploads/2022/06/real-world-phev-use-jun22-1.pdf
 
         Real-world range in simulation 20 km 30 km 40 km 50 km 60 km 70 km 80 km
         Observed UF for Germany (Sample-size weighted regression ± 2 standard errors)
         Observed UF private (in %) 30±2 41±2 50±3 58±3 65±3 71±3 75±3
 
-        which correlated the share of km driven in electric-mode to the capacity of the battery
+        which correlated the share of km driven in electric-mode to
+        the capacity of the battery
         (the range that can be driven in battery-depleting mode).
 
         The argument `uf` is used to override this relation, if needed.
         `uf` must be a ratio between 0 and .75, for each."""
 
         if "PHEV-e" in self.array.coords["powertrain"].values:
-            if uf is None:
+            if self.electric_utility_factor is None:
                 self.array.loc[
                     dict(powertrain="PHEV-e", parameter="electric utility factor")
                 ] = np.clip(
                     np.interp(
                         self.array.loc[dict(powertrain="PHEV-e", parameter="range")],
-                        [0, 20, 30, 40, 50, 60, 70, 80],
-                        [0, 0.3, 0.41, 0.5, 0.58, 0.65, 0.71, 0.75],
+                        [0, 20, 30, 40, 50, 60, 70, 80, 100, 120, 200],
+                        [0, 0.13, 0.18, 0.23, 0.28, 0.30, 0.35, 0.40, 0.45, 0.5, 0.75],
                     ),
                     0,
                     0.75,
                 )
             else:
-                self.array.loc[
-                    dict(powertrain="PHEV-e", parameter="electric utility factor")
-                ] = np.array(uf).reshape([1, -1, 1])
+                for key, val in self.electric_utility_factor.items():
+                    if (
+                        "PHEV-e" in self.array.powertrain.values
+                        and key in self.array.year.values
+                    ):
+                        self.array.loc[
+                            dict(
+                                powertrain="PHEV-e",
+                                parameter="electric utility factor",
+                                year=key,
+                            )
+                        ] = val
 
     def create_PHEV(self):
         """PHEV-p/d is the range-weighted average between PHEV-c-p/PHEV-c-d and PHEV-e."""
@@ -1140,6 +1277,180 @@ class CarModel:
             dict(parameter="energy battery mass")
         ] * (1 - self.array.loc[dict(parameter="battery cell mass share")])
 
+    def set_storage_size_or_range(self):
+        """
+        Set storage size or range for each powertrain.
+        :return:
+        """
+
+        if "capacity" in self.energy_storage:
+            for key, val in self.energy_storage["capacity"].items():
+                pwt, size, year = key
+                if val:
+                    self.array.loc[
+                        dict(
+                            parameter="battery cell mass",
+                            powertrain=pwt,
+                            size=size,
+                            year=year,
+                        )
+                    ] = (
+                        val
+                        / self.array.loc[
+                            dict(
+                                parameter="battery cell energy density",
+                                powertrain=pwt,
+                                size=size,
+                                year=year,
+                            )
+                        ]
+                    )
+
+                    self.array.loc[
+                        dict(
+                            parameter="energy battery mass",
+                            powertrain=pwt,
+                            size=size,
+                            year=year,
+                        )
+                    ] = (
+                        self.array.loc[
+                            dict(
+                                parameter="battery cell mass",
+                                powertrain=pwt,
+                                size=size,
+                                year=year,
+                            )
+                        ]
+                        / self.array.loc[
+                            dict(
+                                parameter="battery cell mass share",
+                                powertrain=pwt,
+                                size=size,
+                                year=year,
+                            )
+                        ]
+                    )
+
+                    self.array.loc[
+                        dict(
+                            parameter="battery BoP mass",
+                            powertrain=pwt,
+                            size=size,
+                            year=year,
+                        )
+                    ] = (
+                        self.array.loc[
+                            dict(
+                                parameter="energy battery mass",
+                                powertrain=pwt,
+                                size=size,
+                                year=year,
+                            )
+                        ]
+                        - self.array.loc[
+                            dict(
+                                parameter="battery cell mass",
+                                powertrain=pwt,
+                                size=size,
+                                year=year,
+                            )
+                        ]
+                    )
+
+        if self.target_range:
+
+            for key, val in self.target_range.items():
+                pwt, size, year = key
+
+                if pwt == "BEV" and val is not None:
+                    battery_DoD = self.array.loc[
+                        dict(
+                            powertrain=pwt,
+                            size=size,
+                            year=year,
+                            parameter="battery DoD",
+                        )
+                    ]  # maximum depth of discharge allowed (80%)
+                    TtW = self.array.loc[
+                        dict(
+                            powertrain=pwt, size=size, year=year, parameter="TtW energy"
+                        )
+                    ]  # kj/km
+
+                    energy_stored = val * (TtW / battery_DoD / 3600)
+
+                    self.array.loc[
+                        dict(
+                            parameter="battery cell mass",
+                            powertrain=pwt,
+                            size=size,
+                            year=year,
+                        )
+                    ] = (
+                        energy_stored
+                        / self.array.loc[
+                            dict(
+                                parameter="battery cell energy density",
+                                powertrain=pwt,
+                                size=size,
+                                year=year,
+                            )
+                        ]
+                    )
+
+                    self.array.loc[
+                        dict(
+                            parameter="energy battery mass",
+                            powertrain=pwt,
+                            size=size,
+                            year=year,
+                        )
+                    ] = (
+                        self.array.loc[
+                            dict(
+                                parameter="battery cell mass",
+                                powertrain=pwt,
+                                size=size,
+                                year=year,
+                            )
+                        ]
+                        / self.array.loc[
+                            dict(
+                                parameter="battery cell mass share",
+                                powertrain=pwt,
+                                size=size,
+                                year=year,
+                            )
+                        ]
+                    )
+
+                    self.array.loc[
+                        dict(
+                            parameter="battery BoP mass",
+                            powertrain=pwt,
+                            size=size,
+                            year=year,
+                        )
+                    ] = (
+                        self.array.loc[
+                            dict(
+                                parameter="energy battery mass",
+                                powertrain=pwt,
+                                size=size,
+                                year=year,
+                            )
+                        ]
+                        - self.array.loc[
+                            dict(
+                                parameter="battery cell mass",
+                                powertrain=pwt,
+                                size=size,
+                                year=year,
+                            )
+                        ]
+                    )
+
     def set_range(self) -> None:
         """
         Calculate range autonomy of vehicles
@@ -1265,27 +1576,28 @@ class CarModel:
                 ]
             )
 
-        if "BEV" in self.array.coords["powertrain"].values:
+        list_electric = [i for i in ["BEV", "PHEV-e"] if i in self.array.powertrain]
 
-            self.array.loc[
-                dict(powertrain=["BEV"], parameter="electric energy stored")
-            ] = (
-                self.array.loc[dict(powertrain=["BEV"], parameter="battery cell mass")]
+        for pwt in list_electric:
+            self.array.loc[dict(powertrain=pwt, parameter="electric energy stored")] = (
+                self.array.loc[dict(powertrain=pwt, parameter="battery cell mass")]
                 * self.array.loc[
-                    dict(powertrain=["BEV"], parameter="battery cell energy density")
+                    dict(powertrain=pwt, parameter="battery cell energy density")
                 ]
             )
 
-        if "PHEV-e" in self.array.coords["powertrain"].values:
-
-            self.array.loc[
-                dict(powertrain="PHEV-e", parameter="electric energy stored")
-            ] = (
-                self.array.loc[dict(powertrain="PHEV-e", parameter="battery cell mass")]
-                * self.array.loc[
-                    dict(powertrain="PHEV-e", parameter="battery cell energy density")
-                ]
-            )
+        if "capacity" in self.energy_storage:
+            for key, val in self.energy_storage["capacity"].items():
+                pwt, size, year = key
+                if val:
+                    self.array.loc[
+                        dict(
+                            powertrain=pwt,
+                            size=size,
+                            year=year,
+                            parameter="electric energy stored",
+                        )
+                    ] = val
 
     def set_costs(self) -> None:
         """
